@@ -121,37 +121,14 @@ DefaultPacketUI::DefaultPacketUI(regina::NPacket* newPacket,
 
 PacketPane::PacketPane(ReginaPart* newPart, NPacket* newPacket,
         QWidget* parent, const char* name) : QVBox(parent, name),
-        part(newPart), frame(0), dirty(false), readWrite(false),
+        part(newPart), frame(0), dirty(false), dirtinessBroken(false),
         emergencyClosure(false), emergencyRefresh(false), isCommitting(false),
         extCut(0), extCopy(0), extPaste(0), extUndo(0), extRedo(0) {
-    // Set up the header and dock/undock button.
-    QHBox* headerBox = new QHBox(this);
+    // Should we allow both read and write?
+    readWrite = part->isReadWrite() && newPacket->isPacketEditable();
 
-    header = new PacketHeader(newPacket, headerBox);
-    headerBox->setStretchFactor(header, 1);
-    QWhatsThis::add(header, i18n("This shows the label of the packet "
-        "being viewed, as well as its packet type."));
-
-    dockUndockBtn = new FlatToolButton(headerBox);
-    dockUndockBtn->setToggleButton(true);
-    dockUndockBtn->setPixmap(BarIcon("attach", ReginaPart::factoryInstance()));
-    dockUndockBtn->setTextLabel(i18n("Dock or undock this packet viewer"));
-    dockUndockBtn->setOn(true);
-    QWhatsThis::add(dockUndockBtn, i18n("Dock or undock this packet viewer.  "
-        "A docked viewer sits within the main window, to the right of "
-        "the packet tree.  An undocked viewer floats in its own window."));
-    connect(dockUndockBtn, SIGNAL(toggled(bool)), this, SLOT(floatPane()));
-
-    // Set up the main interface component.
-    mainUI = PacketManager::createUI(newPacket, this);
-    QWidget* mainUIWidget = mainUI->getInterface();
-    if (mainUIWidget->parent() != this) {
-        mainUIWidget->reparent(this, QPoint(0, 0));
-        mainUIWidget->show();
-    }
-    setStretchFactor(mainUIWidget, 1);
-
-    // Set up the footer buttons and other actions.
+    // Create the actions first, since PacketManager::createUI()
+    // might want to modify them.
     actCommit = new KAction(i18n("Co&mmit"), "button_ok", 0 /* shortcut */,
         this, SLOT(commit()), (KActionCollection*)0, "packet_editor_commit");
     actCommit->setEnabled(false);
@@ -178,6 +155,35 @@ PacketPane::PacketPane(ReginaPart* newPart, NPacket* newPacket,
     actClose->setWhatsThis(i18n("Close this packet viewer.  Any changes "
         "that have not been committed will be discarded."));
 
+    // Set up the header and dock/undock button.
+    QHBox* headerBox = new QHBox(this);
+
+    header = new PacketHeader(newPacket, headerBox);
+    headerBox->setStretchFactor(header, 1);
+    QWhatsThis::add(header, i18n("This shows the label of the packet "
+        "being viewed, as well as its packet type."));
+
+    dockUndockBtn = new FlatToolButton(headerBox);
+    dockUndockBtn->setToggleButton(true);
+    dockUndockBtn->setPixmap(BarIcon("attach", ReginaPart::factoryInstance()));
+    dockUndockBtn->setTextLabel(i18n("Dock or undock this packet viewer"));
+    dockUndockBtn->setOn(true);
+    QWhatsThis::add(dockUndockBtn, i18n("Dock or undock this packet viewer.  "
+        "A docked viewer sits within the main window, to the right of "
+        "the packet tree.  An undocked viewer floats in its own window."));
+    connect(dockUndockBtn, SIGNAL(toggled(bool)), this, SLOT(floatPane()));
+
+    // Set up the main interface component.
+    mainUI = PacketManager::createUI(newPacket, this);
+    QWidget* mainUIWidget = mainUI->getInterface();
+    if (mainUIWidget->parent() != this) {
+        mainUIWidget->reparent(this, QPoint(0, 0));
+        mainUIWidget->show();
+    }
+    setStretchFactor(mainUIWidget, 1);
+    setFocusProxy(mainUIWidget);
+
+    // Set up the footer buttons and other actions.
     KToolBar* footer = new KToolBar(this, "packetEditorBar", false, false);
     footer->setFullSize(true);
     footer->setIconText(KToolBar::IconTextRight);
@@ -237,7 +243,7 @@ PacketPane::~PacketPane() {
 }
 
 void PacketPane::setDirty(bool newDirty) {
-    if (dirty == newDirty)
+    if (dirtinessBroken || dirty == newDirty)
         return;
 
     dirty = newDirty;
@@ -247,6 +253,15 @@ void PacketPane::setDirty(bool newDirty) {
     actRefresh->setIcon(dirty ? "button_cancel" : "reload");
 }
 
+void PacketPane::setDirtinessBroken() {
+    dirtinessBroken = true;
+    dirty = readWrite;
+
+    actCommit->setEnabled(dirty);
+    actRefresh->setText(dirty ? i18n("&Discard / Refresh") : i18n("&Refresh"));
+    actRefresh->setIcon("reload");
+}
+
 bool PacketPane::setReadWrite(bool allowReadWrite) {
     if (allowReadWrite)
         if (! (mainUI->getPacket()->isPacketEditable() && part->isReadWrite()))
@@ -261,6 +276,11 @@ bool PacketPane::setReadWrite(bool allowReadWrite) {
     mainUI->setReadWrite(allowReadWrite);
     updateClipboardActions();
     updateUndoActions();
+    if (dirtinessBroken) {
+        // Update the UI according to the new value of readWrite.
+        setDirtinessBroken();
+    }
+
     emit readWriteStatusChanged(readWrite);
 
     return true;
@@ -268,10 +288,14 @@ bool PacketPane::setReadWrite(bool allowReadWrite) {
 
 bool PacketPane::queryClose() {
     if ((! emergencyClosure) && dirty) {
-        if (KMessageBox::warningContinueCancel(this, i18n(
-                "This packet contains changes that have not yet been "
-                "committed.  Are you sure you wish to close this packet "
-                "now and discard these changes?"),
+        QString msg = (dirtinessBroken ?
+            i18n("This packet might contain changes that have not yet been "
+                 "committed.  Are you sure you wish to close this packet "
+                 "now and discard these changes?") :
+            i18n("This packet contains changes that have not yet been "
+                 "committed.  Are you sure you wish to close this packet "
+                 "now and discard these changes?"));
+        if (KMessageBox::warningContinueCancel(this, msg,
                 mainUI->getPacket()->getPacketLabel().c_str(),
                 KStdGuiItem::close()) == KMessageBox::Cancel)
             return false;
@@ -372,45 +396,74 @@ void PacketPane::deregisterEditOperation(KAction* act, EditOperation op) {
 }
 
 void PacketPane::packetWasChanged(regina::NPacket*) {
+    // Assume it's this packet.
+
     // Ignore this if we're responsible for the event.
     if (isCommitting)
         return;
 
     header->refresh();
 
-    if (dirty)
-        if (KMessageBox::warningYesNo(this, i18n(
-                "This packet has been changed from within a script or "
-                "another interface.  However, this interface contains "
-                "changes that have not yet been committed.  Do you wish "
-                "to refresh this interface to reflect the changes "
-                "that have been made elsewhere?"),
+    if (dirty) {
+        QString msg = (dirtinessBroken ?
+            i18n("This packet has been changed from within a script or "
+                 "another interface.  However, this interface might contain "
+                 "changes that have not yet been committed.  Do you wish "
+                 "to refresh this interface to reflect the changes "
+                 "that have been made elsewhere?") :
+            i18n("This packet has been changed from within a script or "
+                 "another interface.  However, this interface contains "
+                 "changes that have not yet been committed.  Do you wish "
+                 "to refresh this interface to reflect the changes "
+                 "that have been made elsewhere?"));
+        if (KMessageBox::warningYesNo(this, msg,
                 mainUI->getPacket()->getPacketLabel().c_str()) ==
                 KMessageBox::No)
             return;
+    }
 
     mainUI->refresh();
     setDirty(false); // Just in case somebody forgot.
 }
 
 void PacketPane::packetWasRenamed(regina::NPacket*) {
+    // Assume it's this packet.
     header->refresh();
 }
 
 void PacketPane::packetToBeDestroyed(regina::NPacket*) {
+    // Assume it's this packet.
     closeForce();
 }
 
+void PacketPane::childWasAdded(regina::NPacket* packet, regina::NPacket*) {
+    // Assume it's this packet.
+    if (packet->isPacketEditable() != readWrite)
+        setReadWrite(!readWrite);
+}
+
+void PacketPane::childWasRemoved(regina::NPacket* packet, regina::NPacket*) {
+    // Assume it's this packet.
+    if (packet->isPacketEditable() != readWrite)
+        setReadWrite(!readWrite);
+}
+
 void PacketPane::refresh() {
     header->refresh();
 
-    if ((! emergencyRefresh) && dirty)
-        if (KMessageBox::warningContinueCancel(this, i18n(
-                "This packet contains changes that have not yet been "
-                "committed.  Are you sure you wish to discard these changes?"),
+    if ((! emergencyRefresh) && dirty) {
+        QString msg = (dirtinessBroken ?
+            i18n("This packet might contain changes that have not yet been "
+                 "committed.  Are you sure you wish to discard these "
+                 "changes?") :
+            i18n("This packet contains changes that have not yet been "
+                 "committed.  Are you sure you wish to discard these "
+                 "changes?"));
+        if (KMessageBox::warningContinueCancel(this, msg,
                 mainUI->getPacket()->getPacketLabel().c_str(),
                 KStdGuiItem::discard()) != KMessageBox::Continue)
             return;
+    }
 
     emergencyRefresh = false;
     mainUI->refresh();
@@ -422,8 +475,28 @@ void PacketPane::refreshForce() {
     refresh();
 }
 
-void PacketPane::commit() {
+bool PacketPane::commit() {
     if (dirty) {
+        if (! mainUI->getPacket()->isPacketEditable()) {
+            KMessageBox::sorry(this, i18n("<qt>This packet may not be "
+                "edited at the present time.  Because of this, your "
+                "changes cannot be committed.<p>"
+                "This is generally due to a tight relationship shared "
+                "with some other packet in the tree.  For instance, a "
+                "triangulation containing a normal surface list may "
+                "not be edited, since the normal surfaces are stored "
+                "as coordinates relative to the triangulation.<p>"
+                "As a workaround for this problem, you might wish to try "
+                "cloning this packet and editing the clone instead.</qt>"));
+            return false;
+        }
+
+        if (! readWrite) {
+            KMessageBox::sorry(this, i18n("This packet is read-only.  "
+                "No changes may be committed."));
+            return false;
+        }
+
         isCommitting = true;
 
         {
@@ -434,6 +507,74 @@ void PacketPane::commit() {
         setDirty(false); // Just in case somebody forgot.
         isCommitting = false;
     }
+
+    return true;
+}
+
+bool PacketPane::commitToModify() {
+    if (! mainUI->getPacket()->isPacketEditable()) {
+        KMessageBox::sorry(this, i18n("<qt>This packet may not be "
+            "modified at the present time.<p>"
+            "This is generally due to a tight relationship shared "
+            "with some other packet in the tree.  For instance, a "
+            "triangulation containing a normal surface list may "
+            "not be edited, since the normal surfaces are stored "
+            "as coordinates relative to the triangulation.<p>"
+            "As a workaround for this problem, you might wish to try "
+            "cloning this packet and editing the clone instead.</qt>"));
+        return false;
+    }
+
+    if (! readWrite) {
+        KMessageBox::sorry(this, i18n("This packet is read-only, and "
+            "so may not be modified."));
+        return false;
+    }
+
+    return commit();
+}
+
+bool PacketPane::tryCommit() {
+    if (dirty) {
+        if (! mainUI->getPacket()->isPacketEditable()) {
+            if (KMessageBox::warningContinueCancel(this,
+                    i18n("<qt>This packet may not be edited at the present "
+                    "time.  Because of this I cannot commit your recent "
+                    "changes, and I will have to work from an old copy "
+                    "of the packet instead.<p>"
+                    "This is generally due to a tight relationship shared "
+                    "with some other packet in the tree.  For instance, a "
+                    "triangulation containing a normal surface list may "
+                    "not be edited, since the normal surfaces are stored "
+                    "as coordinates relative to the triangulation.<p>"
+                    "Do you wish to continue this operation using an old "
+                    "copy of the packet?</qt>"))
+                    != KMessageBox::Continue)
+                return false;
+        } else if (! readWrite) {
+            if (KMessageBox::warningContinueCancel(this,
+                    i18n("<qt>This packet is read-only, but you appear "
+                    "to have made changes that have not yet been committed.  "
+                    "I cannot commit these changes for you, and so I will "
+                    "have to work from an old copy of the packet instead.<p>"
+                    "Do you wish to continue this operation using an old "
+                    "copy of the packet?</qt>"))
+                    != KMessageBox::Continue)
+                return false;
+        } else {
+            isCommitting = true;
+
+            {
+                NPacket::ChangeEventBlock block(mainUI->getPacket());
+                mainUI->commit();
+            }
+
+            setDirty(false); // Just in case somebody forgot.
+            isCommitting = false;
+        }
+    }
+
+    return true;
 }
 
 bool PacketPane::close() {
