@@ -26,8 +26,10 @@
 
 /* end stub */
 
+#include "regina-config.h"
 #include "surfaces/nnormalsurfacelist.h"
 
+#include "examplesaction.h"
 #include "reginaabout.h"
 #include "reginafilter.h"
 #include "reginamain.h"
@@ -38,6 +40,7 @@
 #include <qvbox.h>
 #include <kaccel.h>
 #include <kaction.h>
+#include <kactionclasses.h>
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kedittoolbar.h>
@@ -238,6 +241,21 @@ bool ReginaMain::openURL(const QString& url) {
     return openURL(KURL(url));
 }
 
+bool ReginaMain::openExample(const KURL& url) {
+    // Same as openURL(), but give a pleasant message if the file
+    // doesn't seem to exist.
+    QFile file(url.path());
+    if (! file.exists()) {
+        KMessageBox::sorry(this, i18n("<qt>The example file %1 "
+            "could not be found.<p>Example files should be installed in the "
+            "directory <i>%2</i>.  It appears that they have not been "
+            "installed properly.  Please contact <i>%3</i> for assistance.").
+            arg(url.fileName()).arg(url.directory()).arg(PACKAGE_BUGREPORT));
+        return false;
+    }
+    return openURL(url);
+}
+
 void ReginaMain::pythonConsole() {
     consoles.launchPythonConsole(this, &globalPrefs);
 }
@@ -293,6 +311,11 @@ void ReginaMain::optionsConfigureToolbars() {
     dlg.exec();
 }
 
+void ReginaMain::optionsConfigureEditor() {
+    ReginaEditorChooser dlg(this);
+    dlg.exec();
+}
+
 void ReginaMain::optionsPreferences() {
     ReginaPreferences dlg(this);
     dlg.exec();
@@ -340,6 +363,9 @@ void ReginaMain::setupActions() {
     KStdAction::open(this, SLOT(fileOpen()), actionCollection());
     fileOpenRecent = KStdAction::openRecent(this, SLOT(openURL(const KURL&)),
         actionCollection());
+    fileOpenExample = new ExamplesAction(this, SLOT(openExample(const KURL&)),
+        actionCollection(), "file_open_example");
+    fillExamples();
     KStdAction::close(this, SLOT(close()), actionCollection());
     KStdAction::quit(kapp, SLOT(closeAllWindows()), actionCollection());
 
@@ -352,6 +378,9 @@ void ReginaMain::setupActions() {
     */
 
     // Preferences:
+    act = new KAction(i18n("Choose Text &Editor..."), "configure", 0,
+        this, SLOT(optionsConfigureEditor()), actionCollection(),
+        "configure_editor");
     KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()),
         actionCollection());
     KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()),
@@ -384,6 +413,25 @@ void ReginaMain::setupActions() {
     createGUI(0);
 }
 
+void ReginaMain::fillExamples() {
+    fileOpenExample->addURL("sample-misc.rga",
+        i18n("Introductory Examples"));
+    fileOpenExample->addURL("closed-hyp-census.rga",
+        i18n("Closed Hyperbolic Census"));
+    fileOpenExample->addURL("closed-or-census.rga",
+        i18n("Closed Orientable Census"));
+    fileOpenExample->addURL("closed-nor-census.rga",
+        i18n("Closed Non-Orientable Census"));
+    fileOpenExample->addURL("snappea-census.rga",
+        i18n("Cusped Hyperbolic Census"));
+    fileOpenExample->addURL("knot-link-census.rga",
+        i18n("Knot / Link Complements"));
+    fileOpenExample->addURL("sig-3mfd-census.rga",
+        i18n("Splitting Surface Sigs (General)"));
+    fileOpenExample->addURL("sig-prime-min-census.rga",
+        i18n("Splitting Surface Sigs (Prime, Minimal)"));
+}
+
 void ReginaMain::addRecentFile() {
     if (currentPart && ! currentPart->url().isEmpty()) {
         fileOpenRecent->addURL(currentPart->url());
